//! Expression module

#![allow(warnings)]

mod expr;
mod opcode;
mod parser;
mod program;
mod runtime;
mod value;

#[derive(
    Debug,
    Clone,
    Copy,
    PartialEq,
    Eq,
    Hash,
    derive_more::Display,
    derive_more::Deref,
    derive_more::DerefMut,
    derive_more::From,
    derive_more::Into,
)]
pub struct Register(usize);

#[derive(
    Debug,
    Clone,
    Copy,
    PartialEq,
    Eq,
    Hash,
    derive_more::Display,
    derive_more::Deref,
    derive_more::DerefMut,
    derive_more::From,
    derive_more::Into,
)]
pub struct Addr(usize);

impl Addr {
    pub fn next(&mut self) {
        self.0 += 1;
    }

    pub fn jump_to(&mut self, addr: &Addr) {
        self.0 = addr.0;
    }
}

#[derive(thiserror::Error, Debug)]
pub enum Error {
    #[error("Program is out of bound")]
    OutOfBoundaries,

    #[error("Register {0} is out of boundaries")]
    RegisterOutOfBoundaries(Register),

    #[error("Empty registers")]
    EmptyRegisters,

    #[error("Value is not a number")]
    NotANumber,

    #[error("Math overflow")]
    Overflow,

    #[error("Unexpected rule: {0:?}")]
    UnexpectedRule(Rule),

    #[error("Parsing error: {0:?}")]
    ParsingError(#[from] pest::error::Error<Rule>),

    #[error("Internal parsing error: Missing next rule")]
    MissingNextRule,

    #[error("Error parsing number: {0}")]
    ParseIntError(#[from] ParseIntError),

    #[error("Invalid order: {0}")]
    InvalidOrder(String),

    #[error("Unknown label: {0}")]
    UnknownLabel(usize),

    #[error("Unexpected expression state")]
    UnexpectedExprState,
}

use std::{
    num::ParseIntError,
    ops::{Deref, DerefMut},
};

use parser::Rule;

pub use self::program::Program;
